#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .UnresolvedAddressDto import UnresolvedAddressDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class AccountMetadataTransactionBodyBuilder:
    """Binary layout for an account metadata transaction.

    Attributes:
        target_address: Metadata target address.
        scoped_metadata_key: Metadata key scoped to source, target and type.
        value_size_delta: Change in value size in bytes.
        value: Difference between existing value and new value \note when there is no existing value, new value is same this value \note when there is an existing value, new value is calculated as xor(previous-value, value).
    """
    def __init__(self):
        """ Constructor."""
        self.target_address = bytes(24)
        self.scoped_metadata_key = int()
        self.value_size_delta = int()
        self.value = bytes()

    @classmethod
    def load_from_binary(cls, payload: bytes) -> AccountMetadataTransactionBodyBuilder:
        """Creates an instance of AccountMetadataTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AccountMetadataTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        target_address_ = UnresolvedAddressDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        target_address = target_address_.unresolvedAddress
        bytes_ = bytes_[target_address_.get_size():]
        scoped_metadata_key = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        value_size_delta = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 2))  # kind:SIMPLE
        bytes_ = bytes_[2:]
        value_size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 2))  # kind:SIZE_FIELD
        bytes_ = bytes_[2:]
        value = GeneratorUtils.get_bytes(bytes_, value_size)  # kind:BUFFER
        bytes_ = bytes_[value_size:]

        # create object and call
        result = AccountMetadataTransactionBodyBuilder()
        result.target_address = target_address
        result.scoped_metadata_key = scoped_metadata_key
        result.value_size_delta = value_size_delta
        result.value = value
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += UnresolvedAddressDto(self.target_address).get_size()
        size += 8  # scoped_metadata_key
        size += 2  # value_size_delta
        size += 2  # value_size
        size += len(self.value)
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedAddressDto(self.target_address).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.scoped_metadata_key, 8))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.value_size_delta, 2))  # serial_kind:SIMPLE
        size_value = len(self.value)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 2))  # kind:SIZE_FIELD
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.value)  # kind:BUFFER
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('target_address', to_hex_string(UnresolvedAddressDto(self.target_address).serialize()))
        result += '{:24s} : {}\n'.format('scoped_metadata_key', to_hex_string(GeneratorUtils.uint_to_buffer(self.scoped_metadata_key, 8)))
        result += '{:24s} : {}\n'.format('value_size_delta', to_hex_string(GeneratorUtils.uint_to_buffer(self.value_size_delta, 2)))
        size_value = len(self.value)
        result += '{:24s} : {}\n'.format('value_size', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 2)))
        result += '{:24s} : {}\n'.format('value', to_hex_string(self.value))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .FinalizationEpochDto import FinalizationEpochDto
from .FinalizationPointDto import FinalizationPointDto

class FinalizationRoundBuilder:
    """Binary layout for finalization round.

    Attributes:
        epoch: Finalization epoch.
        point: Finalization point.
    """

    def __init__(self, epoch: FinalizationEpochDto, point: FinalizationPointDto):
        """Constructor.
        Args:
            epoch: Finalization epoch.
            point: Finalization point.
        """
        self.epoch = epoch
        self.point = point


    @classmethod
    def load_from_binary(cls, payload: bytes) -> FinalizationRoundBuilder:
        """Creates an instance of FinalizationRoundBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of FinalizationRoundBuilder.
        """
        bytes_ = bytes(payload)

        epoch = FinalizationEpochDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[epoch.get_size():]
        point = FinalizationPointDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[point.get_size():]
        return FinalizationRoundBuilder(epoch, point)

    def get_epoch(self) -> FinalizationEpochDto:
        """Gets finalization epoch.
        Returns:
            Finalization epoch.
        """
        return self.epoch

    def get_point(self) -> FinalizationPointDto:
        """Gets finalization point.
        Returns:
            Finalization point.
        """
        return self.point

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.epoch.get_size()
        size += self.point.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.epoch.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.point.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AmountDto import AmountDto
from .Hash256Dto import Hash256Dto

class ImportanceBlockFooterBuilder:
    """Binary layout for an importance block footer.

    Attributes:
        votingEligibleAccountsCount: Number of voting eligible accounts.
        harvestingEligibleAccountsCount: Number of harvesting eligible accounts.
        totalVotingBalance: Total balance eligible for voting.
        previousImportanceBlockHash: Previous importance block hash.
    """

    def __init__(self, votingEligibleAccountsCount: int, harvestingEligibleAccountsCount: int, totalVotingBalance: AmountDto, previousImportanceBlockHash: Hash256Dto):
        """Constructor.
        Args:
            votingEligibleAccountsCount: Number of voting eligible accounts.
            harvestingEligibleAccountsCount: Number of harvesting eligible accounts.
            totalVotingBalance: Total balance eligible for voting.
            previousImportanceBlockHash: Previous importance block hash.
        """
        self.votingEligibleAccountsCount = votingEligibleAccountsCount
        self.harvestingEligibleAccountsCount = harvestingEligibleAccountsCount
        self.totalVotingBalance = totalVotingBalance
        self.previousImportanceBlockHash = previousImportanceBlockHash


    @classmethod
    def load_from_binary(cls, payload: bytes) -> ImportanceBlockFooterBuilder:
        """Creates an instance of ImportanceBlockFooterBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of ImportanceBlockFooterBuilder.
        """
        bytes_ = bytes(payload)

        votingEligibleAccountsCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        harvestingEligibleAccountsCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        totalVotingBalance = AmountDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[totalVotingBalance.get_size():]
        previousImportanceBlockHash = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[previousImportanceBlockHash.get_size():]
        return ImportanceBlockFooterBuilder(votingEligibleAccountsCount, harvestingEligibleAccountsCount, totalVotingBalance, previousImportanceBlockHash)

    def get_voting_eligible_accounts_count(self) -> int:
        """Gets number of voting eligible accounts.
        Returns:
            Number of voting eligible accounts.
        """
        return self.votingEligibleAccountsCount

    def get_harvesting_eligible_accounts_count(self) -> int:
        """Gets number of harvesting eligible accounts.
        Returns:
            Number of harvesting eligible accounts.
        """
        return self.harvestingEligibleAccountsCount

    def get_total_voting_balance(self) -> AmountDto:
        """Gets total balance eligible for voting.
        Returns:
            Total balance eligible for voting.
        """
        return self.totalVotingBalance

    def get_previous_importance_block_hash(self) -> Hash256Dto:
        """Gets previous importance block hash.
        Returns:
            Previous importance block hash.
        """
        return self.previousImportanceBlockHash

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 4  # votingEligibleAccountsCount
        size += 8  # harvestingEligibleAccountsCount
        size += self.totalVotingBalance.get_size()
        size += self.previousImportanceBlockHash.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_voting_eligible_accounts_count(), 4))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_harvesting_eligible_accounts_count(), 8))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.totalVotingBalance.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.previousImportanceBlockHash.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
from typing import List
import re
from .GeneratorUtils import GeneratorUtils
from .CosignatureBuilder import CosignatureBuilder
from .EmbeddedTransactionBuilder import EmbeddedTransactionBuilder
from .EmbeddedTransactionBuilderFactory import EmbeddedTransactionBuilderFactory
from .Hash256Dto import Hash256Dto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class AggregateTransactionBodyBuilder:
    """Binary layout for an aggregate transaction.

    Attributes:
        transactions_hash: Aggregate hash of an aggregate's transactions.
        transactions: Sub-transaction data (transactions are variable sized and payload size is in bytes).
        cosignatures: Cosignatures data (fills remaining body space after transactions).
    """
    def __init__(self):
        """ Constructor."""
        self.transactions_hash = bytes(32)
        self.transactions = []
        self.cosignatures = []

    @staticmethod
    def _load_embedded_transactions(transactions, payload: bytes, payload_size: int):
        remaining_byte_sizes = payload_size
        while remaining_byte_sizes > 0:
            item = EmbeddedTransactionBuilderFactory.create_from_payload(payload)
            transactions.append(item)
            item_size = item.get_size() + GeneratorUtils.get_transaction_padding_size(item.get_size(), 8)
            remaining_byte_sizes -= item_size
            payload = payload[item_size:]
        return payload

    @classmethod
    def load_from_binary(cls, payload: bytes) -> AggregateTransactionBodyBuilder:
        """Creates an instance of AggregateTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AggregateTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        transactions_hash_ = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        transactions_hash = transactions_hash_.hash256
        bytes_ = bytes_[transactions_hash_.get_size():]
        payload_size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIZE_FIELD
        bytes_ = bytes_[4:]
        aggregate_transaction_header__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        transactions = []
        bytes_ = AggregateTransactionBodyBuilder._load_embedded_transactions(transactions, bytes_, payload_size)
        cosignatures_ = []
        bytes_ = GeneratorUtils.load_from_binary(CosignatureBuilder, cosignatures_, bytes_, len(bytes_))
        cosignatures = list(map(lambda e: e.as_tuple(), cosignatures_))

        # create object and call
        result = AggregateTransactionBodyBuilder()
        result.transactions_hash = transactions_hash
        result.transactions = transactions
        result.cosignatures = cosignatures
        return result

    @classmethod
    def _serialize_aligned(cls, transaction: EmbeddedTransactionBuilder) -> bytes:
        """Serializes an embeded transaction with correct padding.
        Returns:
            Serialized embedded transaction.
        """
        bytes_ = transaction.serialize()
        padding = bytes(GeneratorUtils.get_transaction_padding_size(len(bytes_), 8))
        return GeneratorUtils.concat_typed_arrays(bytes_, padding)

    @classmethod
    def _get_size_aligned(cls, transaction: EmbeddedTransactionBuilder) -> int:
        """Serializes an embeded transaction with correct padding.
        Returns:
            Serialized embedded transaction.
        """
        size = transaction.get_size()
        paddingSize = GeneratorUtils.get_transaction_padding_size(size, 8)
        return size + paddingSize
    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += Hash256Dto(self.transactions_hash).get_size()
        size += 4  # payload_size
        size += 4  # aggregate_transaction_header__reserved1
        for _ in self.transactions:
            size += self._get_size_aligned(_)
        for _ in self.cosignatures:
            size += CosignatureBuilder.from_tuple(_).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, Hash256Dto(self.transactions_hash).serialize())  # kind:CUSTOM
        # calculate payload size
        size_value = 0
        for _ in self.transactions:
            size_value += self._get_size_aligned(_)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 4))  # kind:SIZE_FIELD
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        for _ in self.transactions: # kind:VAR_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self._serialize_aligned(_))
        for _ in self.cosignatures: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, CosignatureBuilder.from_tuple(_).serialize())
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('transactions_hash', to_hex_string(Hash256Dto(self.transactions_hash).serialize()))
        # calculate payload size
        size_value = 0
        for _ in self.transactions:
            size_value += self._get_size_aligned(_)
        result += '{:24s} : {}\n'.format('payload_size', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 4)))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        for subtransaction in self.transactions: # kind:VAR_ARRAY
            result += ''.join(map(lambda e: '  ' + e + '\n', subtransaction.__str__().split('\n')))
            size = subtransaction.get_size()
            paddingSize = GeneratorUtils.get_transaction_padding_size(size, 8)
            result += '  {:24s} : {} (len: {})\n'.format('<padding>', to_hex_string(bytes(paddingSize)), paddingSize)
        result += '{:24s} : [\n'.format('cosignatures')
        for _ in self.cosignatures: # kind:ARRAY|FILL_ARRAY
            result += '  {}\n'.format(CosignatureBuilder.from_tuple(_).__str__())
        result += ']\n'
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

# pylint: disable=R0911,R0912

# Imports for creating transaction builders
from .TransactionBuilder import TransactionBuilder
from .AccountAddressRestrictionTransactionBuilder import AccountAddressRestrictionTransactionBuilder
from .AccountKeyLinkTransactionBuilder import AccountKeyLinkTransactionBuilder
from .AccountMetadataTransactionBuilder import AccountMetadataTransactionBuilder
from .AccountMosaicRestrictionTransactionBuilder import AccountMosaicRestrictionTransactionBuilder
from .AccountOperationRestrictionTransactionBuilder import AccountOperationRestrictionTransactionBuilder
from .AddressAliasTransactionBuilder import AddressAliasTransactionBuilder
from .AggregateBondedTransactionBuilder import AggregateBondedTransactionBuilder
from .AggregateCompleteTransactionBuilder import AggregateCompleteTransactionBuilder
from .HashLockTransactionBuilder import HashLockTransactionBuilder
from .MosaicAddressRestrictionTransactionBuilder import MosaicAddressRestrictionTransactionBuilder
from .MosaicAliasTransactionBuilder import MosaicAliasTransactionBuilder
from .MosaicDefinitionTransactionBuilder import MosaicDefinitionTransactionBuilder
from .MosaicGlobalRestrictionTransactionBuilder import MosaicGlobalRestrictionTransactionBuilder
from .MosaicMetadataTransactionBuilder import MosaicMetadataTransactionBuilder
from .MosaicSupplyChangeTransactionBuilder import MosaicSupplyChangeTransactionBuilder
from .MultisigAccountModificationTransactionBuilder import MultisigAccountModificationTransactionBuilder
from .NamespaceMetadataTransactionBuilder import NamespaceMetadataTransactionBuilder
from .NamespaceRegistrationTransactionBuilder import NamespaceRegistrationTransactionBuilder
from .NodeKeyLinkTransactionBuilder import NodeKeyLinkTransactionBuilder
from .SecretLockTransactionBuilder import SecretLockTransactionBuilder
from .SecretProofTransactionBuilder import SecretProofTransactionBuilder
from .TransferTransactionBuilder import TransferTransactionBuilder
from .VotingKeyLinkTransactionBuilder import VotingKeyLinkTransactionBuilder
from .VrfKeyLinkTransactionBuilder import VrfKeyLinkTransactionBuilder


class TransactionBuilderFactory:
    """Factory in charge of creating the specific transaction builder from the binary payload.
    """

    @classmethod
    def create_from_payload(cls, payload) -> TransactionBuilder:
        """
        It creates the specific transaction builder from the payload bytes.
        Args:
            payload: bytes
        Returns:
            the TransactionBuilder subclass
        """
        headerBuilder = TransactionBuilder.load_from_binary(payload)
        entityType = headerBuilder.type
        entityTypeVersion = headerBuilder.version
        if entityType == 0x414c and entityTypeVersion == 1:
            return AccountKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x424c and entityTypeVersion == 1:
            return NodeKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4141 and entityTypeVersion == 1:
            return AggregateCompleteTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4241 and entityTypeVersion == 1:
            return AggregateBondedTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4143 and entityTypeVersion == 1:
            return VotingKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4243 and entityTypeVersion == 1:
            return VrfKeyLinkTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4148 and entityTypeVersion == 1:
            return HashLockTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4152 and entityTypeVersion == 1:
            return SecretLockTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4252 and entityTypeVersion == 1:
            return SecretProofTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4144 and entityTypeVersion == 1:
            return AccountMetadataTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4244 and entityTypeVersion == 1:
            return MosaicMetadataTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4344 and entityTypeVersion == 1:
            return NamespaceMetadataTransactionBuilder.load_from_binary(payload)
        if entityType == 0x414d and entityTypeVersion == 1:
            return MosaicDefinitionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x424d and entityTypeVersion == 1:
            return MosaicSupplyChangeTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4155 and entityTypeVersion == 1:
            return MultisigAccountModificationTransactionBuilder.load_from_binary(payload)
        if entityType == 0x424e and entityTypeVersion == 1:
            return AddressAliasTransactionBuilder.load_from_binary(payload)
        if entityType == 0x434e and entityTypeVersion == 1:
            return MosaicAliasTransactionBuilder.load_from_binary(payload)
        if entityType == 0x414e and entityTypeVersion == 1:
            return NamespaceRegistrationTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4150 and entityTypeVersion == 1:
            return AccountAddressRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4250 and entityTypeVersion == 1:
            return AccountMosaicRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4350 and entityTypeVersion == 1:
            return AccountOperationRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4251 and entityTypeVersion == 1:
            return MosaicAddressRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4151 and entityTypeVersion == 1:
            return MosaicGlobalRestrictionTransactionBuilder.load_from_binary(payload)
        if entityType == 0x4154 and entityTypeVersion == 1:
            return TransferTransactionBuilder.load_from_binary(payload)
        return headerBuilder


    @classmethod
    def create_by_name(cls, transaction_name, signer_public_key, network) -> TransactionBuilder:
        """
        It creates the specific transaction builder given name, network and signer.
        Args:
            transaction_name: transaction name
            signer_public_key: signer
            network: network
        Returns:
            the TransactionBuilder subclass
        """
        mapping = {
            'accountKeyLink': AccountKeyLinkTransactionBuilder,
            'nodeKeyLink': NodeKeyLinkTransactionBuilder,
            'aggregateComplete': AggregateCompleteTransactionBuilder,
            'aggregateBonded': AggregateBondedTransactionBuilder,
            'votingKeyLink': VotingKeyLinkTransactionBuilder,
            'vrfKeyLink': VrfKeyLinkTransactionBuilder,
            'hashLock': HashLockTransactionBuilder,
            'secretLock': SecretLockTransactionBuilder,
            'secretProof': SecretProofTransactionBuilder,
            'accountMetadata': AccountMetadataTransactionBuilder,
            'mosaicMetadata': MosaicMetadataTransactionBuilder,
            'namespaceMetadata': NamespaceMetadataTransactionBuilder,
            'mosaicDefinition': MosaicDefinitionTransactionBuilder,
            'mosaicSupplyChange': MosaicSupplyChangeTransactionBuilder,
            'multisigAccountModification': MultisigAccountModificationTransactionBuilder,
            'addressAlias': AddressAliasTransactionBuilder,
            'mosaicAlias': MosaicAliasTransactionBuilder,
            'namespaceRegistration': NamespaceRegistrationTransactionBuilder,
            'accountAddressRestriction': AccountAddressRestrictionTransactionBuilder,
            'accountMosaicRestriction': AccountMosaicRestrictionTransactionBuilder,
            'accountOperationRestriction': AccountOperationRestrictionTransactionBuilder,
            'mosaicAddressRestriction': MosaicAddressRestrictionTransactionBuilder,
            'mosaicGlobalRestriction': MosaicGlobalRestrictionTransactionBuilder,
            'transfer': TransferTransactionBuilder,
        }
        if transaction_name not in mapping:
            raise ValueError('transaction named {} is not supported'.format(transaction_name))

        return mapping[transaction_name](signer_public_key, network)



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .BlockDurationDto import BlockDurationDto
from .MosaicFlagsDto import MosaicFlagsDto

class MosaicPropertiesBuilder:
    """Binary layout for mosaic properties.

    Attributes:
        flags: Mosaic flags.
        divisibility: Mosaic divisibility.
        duration: Mosaic duration.
    """

    def __init__(self, flags: List[MosaicFlagsDto], divisibility: int, duration: BlockDurationDto):
        """Constructor.
        Args:
            flags: Mosaic flags.
            divisibility: Mosaic divisibility.
            duration: Mosaic duration.
        """
        self.flags = flags
        self.divisibility = divisibility
        self.duration = duration


    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicPropertiesBuilder:
        """Creates an instance of MosaicPropertiesBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicPropertiesBuilder.
        """
        bytes_ = bytes(payload)

        flags = MosaicFlagsDto.bytesToFlags(bytes_, 1)  # kind:FLAGS
        bytes_ = bytes_[1:]
        divisibility = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]
        duration = BlockDurationDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[duration.get_size():]
        return MosaicPropertiesBuilder(flags, divisibility, duration)

    def get_flags(self) -> List[MosaicFlagsDto]:
        """Gets mosaic flags.
        Returns:
            Mosaic flags.
        """
        return self.flags

    def get_divisibility(self) -> int:
        """Gets mosaic divisibility.
        Returns:
            Mosaic divisibility.
        """
        return self.divisibility

    def get_duration(self) -> BlockDurationDto:
        """Gets mosaic duration.
        Returns:
            Mosaic duration.
        """
        return self.duration

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 1  # flags
        size += 1  # divisibility
        size += self.duration.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(MosaicFlagsDto.flagsToInt(self.get_flags()), 1))  # kind:FLAGS
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_divisibility(), 1))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.duration.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .BlockDurationDto import BlockDurationDto
from .Hash256Dto import Hash256Dto
from .LockHashAlgorithmDto import LockHashAlgorithmDto
from .UnresolvedAddressDto import UnresolvedAddressDto
from .UnresolvedMosaicBuilder import UnresolvedMosaicBuilder

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class SecretLockTransactionBodyBuilder:
    """Binary layout for a secret lock transaction.

    Attributes:
        recipient_address: Locked mosaic recipient address.
        secret: Secret.
        mosaic: Locked mosaic.
        duration: Number of blocks for which a lock should be valid.
        hash_algorithm: Hash algorithm.
    """
    def __init__(self):
        """ Constructor."""
        self.recipient_address = bytes(24)
        self.secret = bytes(32)
        self.mosaic = None # UnresolvedMosaicBuilder
        self.duration = BlockDurationDto().blockDuration
        self.hash_algorithm = LockHashAlgorithmDto(0).value

    @classmethod
    def load_from_binary(cls, payload: bytes) -> SecretLockTransactionBodyBuilder:
        """Creates an instance of SecretLockTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of SecretLockTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        recipient_address_ = UnresolvedAddressDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        recipient_address = recipient_address_.unresolvedAddress
        bytes_ = bytes_[recipient_address_.get_size():]
        secret_ = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        secret = secret_.hash256
        bytes_ = bytes_[secret_.get_size():]
        mosaic = UnresolvedMosaicBuilder.load_from_binary(bytes_)  # kind:CUSTOM1_nonbyte
        bytes_ = bytes_[mosaic.get_size():]
        mosaic = mosaic.as_tuple()
        duration_ = BlockDurationDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        duration = duration_.blockDuration
        bytes_ = bytes_[duration_.get_size():]
        hash_algorithm_ = LockHashAlgorithmDto.load_from_binary(bytes_)  # kind:CUSTOM2
        hash_algorithm = hash_algorithm_.value
        bytes_ = bytes_[hash_algorithm_.get_size():]

        # create object and call
        result = SecretLockTransactionBodyBuilder()
        result.recipient_address = recipient_address
        result.secret = secret
        result.mosaic = mosaic
        result.duration = duration
        result.hash_algorithm = hash_algorithm
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += UnresolvedAddressDto(self.recipient_address).get_size()
        size += Hash256Dto(self.secret).get_size()
        size += UnresolvedMosaicBuilder.from_tuple(self.mosaic).get_size()
        size += BlockDurationDto(self.duration).get_size()
        size += LockHashAlgorithmDto(self.hash_algorithm).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedAddressDto(self.recipient_address).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, Hash256Dto(self.secret).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedMosaicBuilder.from_tuple(self.mosaic).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, BlockDurationDto(self.duration).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, LockHashAlgorithmDto(self.hash_algorithm).serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('recipient_address', to_hex_string(UnresolvedAddressDto(self.recipient_address).serialize()))
        result += '{:24s} : {}\n'.format('secret', to_hex_string(Hash256Dto(self.secret).serialize()))
        result += '{:24s} : {}\n'.format('mosaic', to_hex_string(UnresolvedMosaicBuilder.from_tuple(self.mosaic).serialize()))
        result += '{:24s} : {}\n'.format('duration', to_hex_string(BlockDurationDto(self.duration).serialize()))
        result += '{:24s} : {}\n'.format('hash_algorithm', to_hex_string(LockHashAlgorithmDto(self.hash_algorithm).serialize()))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .MosaicIdDto import MosaicIdDto
from .MosaicRestrictionTypeDto import MosaicRestrictionTypeDto

class RestrictionRuleBuilder:
    """Binary layout of restriction rule being applied.

    Attributes:
        referenceMosaicId: Identifier of the mosaic providing the restriction key.
        restrictionValue: Restriction value.
        restrictionType: Restriction type.
    """

    def __init__(self, referenceMosaicId: MosaicIdDto, restrictionValue: int, restrictionType: MosaicRestrictionTypeDto):
        """Constructor.
        Args:
            referenceMosaicId: Identifier of the mosaic providing the restriction key.
            restrictionValue: Restriction value.
            restrictionType: Restriction type.
        """
        self.referenceMosaicId = referenceMosaicId
        self.restrictionValue = restrictionValue
        self.restrictionType = restrictionType


    @classmethod
    def load_from_binary(cls, payload: bytes) -> RestrictionRuleBuilder:
        """Creates an instance of RestrictionRuleBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of RestrictionRuleBuilder.
        """
        bytes_ = bytes(payload)

        referenceMosaicId = MosaicIdDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[referenceMosaicId.get_size():]
        restrictionValue = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        restrictionType = MosaicRestrictionTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[restrictionType.get_size():]
        return RestrictionRuleBuilder(referenceMosaicId, restrictionValue, restrictionType)

    def get_reference_mosaic_id(self) -> MosaicIdDto:
        """Gets identifier of the mosaic providing the restriction key.
        Returns:
            Identifier of the mosaic providing the restriction key.
        """
        return self.referenceMosaicId

    def get_restriction_value(self) -> int:
        """Gets restriction value.
        Returns:
            Restriction value.
        """
        return self.restrictionValue

    def get_restriction_type(self) -> MosaicRestrictionTypeDto:
        """Gets restriction type.
        Returns:
            Restriction type.
        """
        return self.restrictionType

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.referenceMosaicId.get_size()
        size += 8  # restrictionValue
        size += self.restrictionType.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.referenceMosaicId.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_restriction_value(), 8))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.restrictionType.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .BlockDurationDto import BlockDurationDto
from .EmbeddedTransactionBuilder import EmbeddedTransactionBuilder
from .EntityTypeDto import EntityTypeDto
from .KeyDto import KeyDto
from .NamespaceIdDto import NamespaceIdDto
from .NamespaceRegistrationTransactionBodyBuilder import NamespaceRegistrationTransactionBodyBuilder
from .NamespaceRegistrationTypeDto import NamespaceRegistrationTypeDto
from .NetworkTypeDto import NetworkTypeDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class EmbeddedNamespaceRegistrationTransactionBuilder(EmbeddedTransactionBuilder):
    """Binary layout for an embedded namespace registration transaction.

    Attributes:
        body: Namespace registration transaction body.
    """
    type_hints = {
        'embedded_transaction' : 'EmbeddedTransactionBuilder',
        'signer_public_key' : 'KeyDto',
        'network' : 'NetworkTypeDto',
        'type' : 'EntityTypeDto',
        'duration' : 'BlockDurationDto',
        'parent_id' : 'NamespaceIdDto',
        'id' : 'NamespaceIdDto',
        'registration_type' : 'NamespaceRegistrationTypeDto',
    }

    VERSION = 1
    ENTITY_TYPE = 0x414e

    def __init__(self, signer_public_key, network: NetworkTypeDto):
        """Constructor.
        Args:
            signer_public_key: Entity signer's public key.
            network: Entity network.
        """
        super().__init__(signer_public_key, self.VERSION, network, self.ENTITY_TYPE)

        self.body = NamespaceRegistrationTransactionBodyBuilder()

    @classmethod
    def load_from_binary(cls, payload: bytes) -> EmbeddedNamespaceRegistrationTransactionBuilder:
        """Creates an instance of EmbeddedNamespaceRegistrationTransactionBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of EmbeddedNamespaceRegistrationTransactionBuilder.
        """
        bytes_ = bytes(payload)
        superObject = EmbeddedTransactionBuilder.load_from_binary(bytes_)
        assert cls.VERSION == superObject.version, 'Invalid entity version ({})'.format(superObject.version)
        assert cls.ENTITY_TYPE == superObject.type, 'Invalid entity type ({})'.format(superObject.type)
        bytes_ = bytes_[superObject.get_size():]

        body = NamespaceRegistrationTransactionBodyBuilder.load_from_binary(bytes_)  # kind:CUSTOM1_nonbyte
        bytes_ = bytes_[body.get_size():]

        # create object and call
        result = EmbeddedNamespaceRegistrationTransactionBuilder(superObject.signer_public_key, superObject.network)
        # nothing needed to copy into EmbeddedTransaction
        result.body = body
        return result

    @property
    def duration(self):
        return self.body.duration

    @duration.setter
    def duration(self, duration): # MARKER1 AttributeKind.CUSTOM
        self.body.duration = duration

    @property
    def parent_id(self):
        return self.body.parent_id

    @parent_id.setter
    def parent_id(self, parent_id): # MARKER1 AttributeKind.CUSTOM
        self.body.parent_id = parent_id

    @property
    def id(self):
        return self.body.id

    @id.setter
    def id(self, id): # MARKER1 AttributeKind.CUSTOM
        self.body.id = id

    @property
    def registration_type(self):
        return self.body.registration_type

    @registration_type.setter
    def registration_type(self, registration_type): # MARKER1 AttributeKind.CUSTOM
        self.body.registration_type = registration_type

    @property
    def name(self):
        return self.body.name

    @name.setter
    def name(self, name): # MARKER1 AttributeKind.BUFFER
        self.body.name = name

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.body.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.body.serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += super().__str__()
        result += self.body.__str__()
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from enum import Enum
from .GeneratorUtils import GeneratorUtils


class ReceiptTypeDto(Enum):
    """Enumeration of receipt types

    Attributes:
        RESERVED: reserved receipt type.
        MOSAIC_RENTAL_FEE: mosaic rental fee receipt type.
        NAMESPACE_RENTAL_FEE: namespace rental fee receipt type.
        HARVEST_FEE: harvest fee receipt type.
        LOCK_HASH_COMPLETED: lock hash completed receipt type.
        LOCK_HASH_EXPIRED: lock hash expired receipt type.
        LOCK_SECRET_COMPLETED: lock secret completed receipt type.
        LOCK_SECRET_EXPIRED: lock secret expired receipt type.
        LOCK_HASH_CREATED: lock hash created receipt type.
        LOCK_SECRET_CREATED: lock secret created receipt type.
        MOSAIC_EXPIRED: mosaic expired receipt type.
        NAMESPACE_EXPIRED: namespace expired receipt type.
        NAMESPACE_DELETED: namespace deleted receipt type.
        INFLATION: inflation receipt type.
        TRANSACTION_GROUP: transaction group receipt type.
        ADDRESS_ALIAS_RESOLUTION: address alias resolution receipt type.
        MOSAIC_ALIAS_RESOLUTION: mosaic alias resolution receipt type.
    """

    RESERVED = 0
    MOSAIC_RENTAL_FEE = 4685
    NAMESPACE_RENTAL_FEE = 4942
    HARVEST_FEE = 8515
    LOCK_HASH_COMPLETED = 8776
    LOCK_HASH_EXPIRED = 9032
    LOCK_SECRET_COMPLETED = 8786
    LOCK_SECRET_EXPIRED = 9042
    LOCK_HASH_CREATED = 12616
    LOCK_SECRET_CREATED = 12626
    MOSAIC_EXPIRED = 16717
    NAMESPACE_EXPIRED = 16718
    NAMESPACE_DELETED = 16974
    INFLATION = 20803
    TRANSACTION_GROUP = 57667
    ADDRESS_ALIAS_RESOLUTION = 61763
    MOSAIC_ALIAS_RESOLUTION = 62019

    @classmethod
    def load_from_binary(cls, payload: bytes) -> ReceiptTypeDto:
        """Creates an instance of ReceiptTypeDto from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of ReceiptTypeDto.
        """
        value: int = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes(payload), 2))
        return ReceiptTypeDto(value)

    @classmethod
    def get_size(cls) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        return 2

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.value, 2))
        return bytes_



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .CosignatureBuilder import CosignatureBuilder
from .Hash256Dto import Hash256Dto
from .KeyDto import KeyDto
from .SignatureDto import SignatureDto

class DetachedCosignatureBuilder(CosignatureBuilder):
    """Cosignature detached from an aggregate transaction.

    Attributes:
        parentHash: Hash of the aggregate transaction that is signed by this cosignature.
    """

    def __init__(self, version: int, signerPublicKey: KeyDto, signature: SignatureDto, parentHash: Hash256Dto):
        """Constructor.
        Args:
            version: Version.
            signerPublicKey: Cosigner public key.
            signature: Cosigner signature.
            parentHash: Hash of the aggregate transaction that is signed by this cosignature.
        """
        super().__init__(version, signerPublicKey, signature)
        self.parentHash = parentHash


    @classmethod
    def load_from_binary(cls, payload: bytes) -> DetachedCosignatureBuilder:
        """Creates an instance of DetachedCosignatureBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of DetachedCosignatureBuilder.
        """
        bytes_ = bytes(payload)
        superObject = CosignatureBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        parentHash = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[parentHash.get_size():]
        return DetachedCosignatureBuilder(superObject.version, superObject.signerPublicKey, superObject.signature, parentHash)

    def get_parent_hash(self) -> Hash256Dto:
        """Gets hash of the aggregate transaction that is signed by this cosignature.
        Returns:
            Hash of the aggregate transaction that is signed by this cosignature.
        """
        return self.parentHash

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.parentHash.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.parentHash.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .ReceiptSourceBuilder import ReceiptSourceBuilder

class AddressResolutionEntryBuilder:
    """Binary layout for address resolution entry.

    Attributes:
        source: Source of resolution within block.
        resolved: Resolved value.
    """

    def __init__(self, source: ReceiptSourceBuilder, resolved: AddressDto):
        """Constructor.
        Args:
            source: Source of resolution within block.
            resolved: Resolved value.
        """
        self.source = source
        self.resolved = resolved


    @classmethod
    def load_from_binary(cls, payload: bytes) -> AddressResolutionEntryBuilder:
        """Creates an instance of AddressResolutionEntryBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AddressResolutionEntryBuilder.
        """
        bytes_ = bytes(payload)

        source = ReceiptSourceBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[source.get_size():]
        resolved = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[resolved.get_size():]
        return AddressResolutionEntryBuilder(source, resolved)

    def get_source(self) -> ReceiptSourceBuilder:
        """Gets source of resolution within block.
        Returns:
            Source of resolution within block.
        """
        return self.source

    def get_resolved(self) -> AddressDto:
        """Gets resolved value.
        Returns:
            Resolved value.
        """
        return self.resolved

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.source.get_size()
        size += self.resolved.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.source.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.resolved.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils

class ReceiptSourceBuilder:
    """Binary layout for receipt source.

    Attributes:
        primaryId: Transaction primary source (e.g. index within block).
        secondaryId: Transaction secondary source (e.g. index within aggregate).
    """

    def __init__(self, primaryId: int, secondaryId: int):
        """Constructor.
        Args:
            primaryId: Transaction primary source (e.g. index within block).
            secondaryId: Transaction secondary source (e.g. index within aggregate).
        """
        self.primaryId = primaryId
        self.secondaryId = secondaryId


    @classmethod
    def load_from_binary(cls, payload: bytes) -> ReceiptSourceBuilder:
        """Creates an instance of ReceiptSourceBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of ReceiptSourceBuilder.
        """
        bytes_ = bytes(payload)

        primaryId = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        secondaryId = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        return ReceiptSourceBuilder(primaryId, secondaryId)

    def get_primary_id(self) -> int:
        """Gets transaction primary source (e.g. index within block).
        Returns:
            Transaction primary source (e.g. index within block).
        """
        return self.primaryId

    def get_secondary_id(self) -> int:
        """Gets transaction secondary source (e.g. index within aggregate).
        Returns:
            Transaction secondary source (e.g. index within aggregate).
        """
        return self.secondaryId

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 4  # primaryId
        size += 4  # secondaryId
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_primary_id(), 4))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_secondary_id(), 4))  # kind:SIMPLE
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .HeightDto import HeightDto
from .MosaicPropertiesBuilder import MosaicPropertiesBuilder

class MosaicDefinitionBuilder:
    """Binary layout for mosaic definition.

    Attributes:
        startHeight: Block height.
        ownerAddress: Mosaic owner.
        revision: Revision.
        properties: Properties.
    """

    def __init__(self, startHeight: HeightDto, ownerAddress: AddressDto, revision: int, properties: MosaicPropertiesBuilder):
        """Constructor.
        Args:
            startHeight: Block height.
            ownerAddress: Mosaic owner.
            revision: Revision.
            properties: Properties.
        """
        self.startHeight = startHeight
        self.ownerAddress = ownerAddress
        self.revision = revision
        self.properties = properties


    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicDefinitionBuilder:
        """Creates an instance of MosaicDefinitionBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicDefinitionBuilder.
        """
        bytes_ = bytes(payload)

        startHeight = HeightDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[startHeight.get_size():]
        ownerAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[ownerAddress.get_size():]
        revision = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        properties = MosaicPropertiesBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[properties.get_size():]
        return MosaicDefinitionBuilder(startHeight, ownerAddress, revision, properties)

    def get_start_height(self) -> HeightDto:
        """Gets block height.
        Returns:
            Block height.
        """
        return self.startHeight

    def get_owner_address(self) -> AddressDto:
        """Gets mosaic owner.
        Returns:
            Mosaic owner.
        """
        return self.ownerAddress

    def get_revision(self) -> int:
        """Gets revision.
        Returns:
            Revision.
        """
        return self.revision

    def get_properties(self) -> MosaicPropertiesBuilder:
        """Gets properties.
        Returns:
            Properties.
        """
        return self.properties

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.startHeight.get_size()
        size += self.ownerAddress.get_size()
        size += 4  # revision
        size += self.properties.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.startHeight.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.ownerAddress.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_revision(), 4))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.properties.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .Hash256Dto import Hash256Dto
from .HeightDto import HeightDto
from .LockHashAlgorithmDto import LockHashAlgorithmDto
from .LockStatusDto import LockStatusDto
from .MosaicBuilder import MosaicBuilder
from .StateHeaderBuilder import StateHeaderBuilder

class SecretLockInfoBuilder(StateHeaderBuilder):
    """Binary layout for serialized lock transaction.

    Attributes:
        ownerAddress: Owner address.
        mosaic: Mosaic associated with lock.
        endHeight: Height at which the lock expires.
        status: Flag indicating whether or not the lock was already used.
        hashAlgorithm: Hash algorithm.
        secret: Transaction secret.
        recipient: Transaction recipient.
    """

    def __init__(self, version: int, ownerAddress: AddressDto, mosaic: MosaicBuilder, endHeight: HeightDto, status: LockStatusDto, hashAlgorithm: LockHashAlgorithmDto, secret: Hash256Dto, recipient: AddressDto):
        """Constructor.
        Args:
            version: Serialization version.
            ownerAddress: Owner address.
            mosaic: Mosaic associated with lock.
            endHeight: Height at which the lock expires.
            status: Flag indicating whether or not the lock was already used.
            hashAlgorithm: Hash algorithm.
            secret: Transaction secret.
            recipient: Transaction recipient.
        """
        super().__init__(version)
        self.ownerAddress = ownerAddress
        self.mosaic = mosaic
        self.endHeight = endHeight
        self.status = status
        self.hashAlgorithm = hashAlgorithm
        self.secret = secret
        self.recipient = recipient


    @classmethod
    def load_from_binary(cls, payload: bytes) -> SecretLockInfoBuilder:
        """Creates an instance of SecretLockInfoBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of SecretLockInfoBuilder.
        """
        bytes_ = bytes(payload)
        superObject = StateHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        ownerAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[ownerAddress.get_size():]
        mosaic = MosaicBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[mosaic.get_size():]
        endHeight = HeightDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[endHeight.get_size():]
        status = LockStatusDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[status.get_size():]
        hashAlgorithm = LockHashAlgorithmDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[hashAlgorithm.get_size():]
        secret = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[secret.get_size():]
        recipient = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[recipient.get_size():]
        return SecretLockInfoBuilder(superObject.version, ownerAddress, mosaic, endHeight, status, hashAlgorithm, secret, recipient)

    def get_owner_address(self) -> AddressDto:
        """Gets owner address.
        Returns:
            Owner address.
        """
        return self.ownerAddress

    def get_mosaic(self) -> MosaicBuilder:
        """Gets mosaic associated with lock.
        Returns:
            Mosaic associated with lock.
        """
        return self.mosaic

    def get_end_height(self) -> HeightDto:
        """Gets height at which the lock expires.
        Returns:
            Height at which the lock expires.
        """
        return self.endHeight

    def get_status(self) -> LockStatusDto:
        """Gets flag indicating whether or not the lock was already used.
        Returns:
            Flag indicating whether or not the lock was already used.
        """
        return self.status

    def get_hash_algorithm(self) -> LockHashAlgorithmDto:
        """Gets hash algorithm.
        Returns:
            Hash algorithm.
        """
        return self.hashAlgorithm

    def get_secret(self) -> Hash256Dto:
        """Gets transaction secret.
        Returns:
            Transaction secret.
        """
        return self.secret

    def get_recipient(self) -> AddressDto:
        """Gets transaction recipient.
        Returns:
            Transaction recipient.
        """
        return self.recipient

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.ownerAddress.get_size()
        size += self.mosaic.get_size()
        size += self.endHeight.get_size()
        size += self.status.get_size()
        size += self.hashAlgorithm.get_size()
        size += self.secret.get_size()
        size += self.recipient.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.ownerAddress.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.mosaic.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.endHeight.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.status.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.hashAlgorithm.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.secret.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.recipient.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .AddressDto import AddressDto
from .BlockFeeMultiplierDto import BlockFeeMultiplierDto
from .DifficultyDto import DifficultyDto
from .EntityTypeDto import EntityTypeDto
from .Hash256Dto import Hash256Dto
from .HeightDto import HeightDto
from .KeyDto import KeyDto
from .NetworkTypeDto import NetworkTypeDto
from .SignatureDto import SignatureDto
from .TimestampDto import TimestampDto
from .VrfProofBuilder import VrfProofBuilder

class BlockHeaderBuilder:
    """Binary layout for a block header.

    Attributes:
        signature: Entity signature.
        signerPublicKey: Entity signer's public key.
        version: Entity version.
        network: Entity network.
        type: Entity type.
        height: Block height.
        timestamp: Number of milliseconds elapsed since creation of nemesis block.
        difficulty: Block difficulty.
        generationHashProof: Generation hash proof.
        previousBlockHash: Previous block hash.
        transactionsHash: Hash of the transactions in this block.
        receiptsHash: Hash of the receipts generated by this block.
        stateHash: Hash of the global chain state at this block.
        beneficiaryAddress: Beneficiary address designated by harvester.
        feeMultiplier: Fee multiplier applied to block transactions.
    """

    def __init__(self, signature: SignatureDto, signerPublicKey: KeyDto, version: int, network: NetworkTypeDto, type: EntityTypeDto, height: HeightDto, timestamp: TimestampDto, difficulty: DifficultyDto, generationHashProof: VrfProofBuilder, previousBlockHash: Hash256Dto, transactionsHash: Hash256Dto, receiptsHash: Hash256Dto, stateHash: Hash256Dto, beneficiaryAddress: AddressDto, feeMultiplier: BlockFeeMultiplierDto):
        """Constructor.
        Args:
            signature: Entity signature.
            signerPublicKey: Entity signer's public key.
            version: Entity version.
            network: Entity network.
            type: Entity type.
            height: Block height.
            timestamp: Number of milliseconds elapsed since creation of nemesis block.
            difficulty: Block difficulty.
            generationHashProof: Generation hash proof.
            previousBlockHash: Previous block hash.
            transactionsHash: Hash of the transactions in this block.
            receiptsHash: Hash of the receipts generated by this block.
            stateHash: Hash of the global chain state at this block.
            beneficiaryAddress: Beneficiary address designated by harvester.
            feeMultiplier: Fee multiplier applied to block transactions.
        """
        self.signature = signature
        self.signerPublicKey = signerPublicKey
        self.version = version
        self.network = network
        self.type = type
        self.height = height
        self.timestamp = timestamp
        self.difficulty = difficulty
        self.generationHashProof = generationHashProof
        self.previousBlockHash = previousBlockHash
        self.transactionsHash = transactionsHash
        self.receiptsHash = receiptsHash
        self.stateHash = stateHash
        self.beneficiaryAddress = beneficiaryAddress
        self.feeMultiplier = feeMultiplier


    @classmethod
    def load_from_binary(cls, payload: bytes) -> BlockHeaderBuilder:
        """Creates an instance of BlockHeaderBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of BlockHeaderBuilder.
        """
        bytes_ = bytes(payload)

        size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        verifiableEntityHeader_Reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        signature = SignatureDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[signature.get_size():]
        signerPublicKey = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[signerPublicKey.get_size():]
        entityBody_Reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        version = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]
        network = NetworkTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[network.get_size():]
        type = EntityTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[type.get_size():]
        height = HeightDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[height.get_size():]
        timestamp = TimestampDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[timestamp.get_size():]
        difficulty = DifficultyDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[difficulty.get_size():]
        generationHashProof = VrfProofBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[generationHashProof.get_size():]
        previousBlockHash = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[previousBlockHash.get_size():]
        transactionsHash = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[transactionsHash.get_size():]
        receiptsHash = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[receiptsHash.get_size():]
        stateHash = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[stateHash.get_size():]
        beneficiaryAddress = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[beneficiaryAddress.get_size():]
        feeMultiplier = BlockFeeMultiplierDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[feeMultiplier.get_size():]
        return BlockHeaderBuilder(signature, signerPublicKey, version, network, type, height, timestamp, difficulty, generationHashProof, previousBlockHash, transactionsHash, receiptsHash, stateHash, beneficiaryAddress, feeMultiplier)

    def get_signature(self) -> SignatureDto:
        """Gets entity signature.
        Returns:
            Entity signature.
        """
        return self.signature

    def get_signer_public_key(self) -> KeyDto:
        """Gets entity signer's public key.
        Returns:
            Entity signer's public key.
        """
        return self.signerPublicKey

    def get_version(self) -> int:
        """Gets entity version.
        Returns:
            Entity version.
        """
        return self.version

    def get_network(self) -> NetworkTypeDto:
        """Gets entity network.
        Returns:
            Entity network.
        """
        return self.network

    def get_type(self) -> EntityTypeDto:
        """Gets entity type.
        Returns:
            Entity type.
        """
        return self.type

    def get_height(self) -> HeightDto:
        """Gets block height.
        Returns:
            Block height.
        """
        return self.height

    def get_timestamp(self) -> TimestampDto:
        """Gets number of milliseconds elapsed since creation of nemesis block.
        Returns:
            Number of milliseconds elapsed since creation of nemesis block.
        """
        return self.timestamp

    def get_difficulty(self) -> DifficultyDto:
        """Gets block difficulty.
        Returns:
            Block difficulty.
        """
        return self.difficulty

    def get_generation_hash_proof(self) -> VrfProofBuilder:
        """Gets generation hash proof.
        Returns:
            Generation hash proof.
        """
        return self.generationHashProof

    def get_previous_block_hash(self) -> Hash256Dto:
        """Gets previous block hash.
        Returns:
            Previous block hash.
        """
        return self.previousBlockHash

    def get_transactions_hash(self) -> Hash256Dto:
        """Gets hash of the transactions in this block.
        Returns:
            Hash of the transactions in this block.
        """
        return self.transactionsHash

    def get_receipts_hash(self) -> Hash256Dto:
        """Gets hash of the receipts generated by this block.
        Returns:
            Hash of the receipts generated by this block.
        """
        return self.receiptsHash

    def get_state_hash(self) -> Hash256Dto:
        """Gets hash of the global chain state at this block.
        Returns:
            Hash of the global chain state at this block.
        """
        return self.stateHash

    def get_beneficiary_address(self) -> AddressDto:
        """Gets beneficiary address designated by harvester.
        Returns:
            Beneficiary address designated by harvester.
        """
        return self.beneficiaryAddress

    def get_fee_multiplier(self) -> BlockFeeMultiplierDto:
        """Gets fee multiplier applied to block transactions.
        Returns:
            Fee multiplier applied to block transactions.
        """
        return self.feeMultiplier

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 4  # size
        size += 4  # verifiableEntityHeader_Reserved1
        size += self.signature.get_size()
        size += self.signerPublicKey.get_size()
        size += 4  # entityBody_Reserved1
        size += 1  # version
        size += self.network.get_size()
        size += self.type.get_size()
        size += self.height.get_size()
        size += self.timestamp.get_size()
        size += self.difficulty.get_size()
        size += self.generationHashProof.get_size()
        size += self.previousBlockHash.get_size()
        size += self.transactionsHash.get_size()
        size += self.receiptsHash.get_size()
        size += self.stateHash.get_size()
        size += self.beneficiaryAddress.get_size()
        size += self.feeMultiplier.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_size(), 4))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.signature.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.signerPublicKey.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.get_version(), 1))  # kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.network.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.type.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.height.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.timestamp.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.difficulty.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.generationHashProof.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.previousBlockHash.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.transactionsHash.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.receiptsHash.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.stateHash.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.beneficiaryAddress.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.feeMultiplier.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .AccountRestrictionAddressValueBuilder import AccountRestrictionAddressValueBuilder
from .AccountRestrictionFlagsDto import AccountRestrictionFlagsDto
from .AccountRestrictionMosaicValueBuilder import AccountRestrictionMosaicValueBuilder
from .AccountRestrictionTransactionTypeValueBuilder import AccountRestrictionTransactionTypeValueBuilder

class AccountRestrictionsInfoBuilder:
    """Binary layout for account restrictions.

    Attributes:
        restrictionFlags: Raw restriction flags.
        addressRestrictions: Address restrictions.
        mosaicIdRestrictions: Mosaic identifier restrictions.
        transactionTypeRestrictions: Transaction type restrictions.
    """

    def __init__(self, restrictionFlags: List[AccountRestrictionFlagsDto], addressRestrictions: AccountRestrictionAddressValueBuilder, mosaicIdRestrictions: AccountRestrictionMosaicValueBuilder, transactionTypeRestrictions: AccountRestrictionTransactionTypeValueBuilder):
        """Constructor.
        Args:
            restrictionFlags: Raw restriction flags.
            addressRestrictions: Address restrictions.
            mosaicIdRestrictions: Mosaic identifier restrictions.
            transactionTypeRestrictions: Transaction type restrictions.
        """
        self.restrictionFlags = restrictionFlags
        self.addressRestrictions = addressRestrictions
        self.mosaicIdRestrictions = mosaicIdRestrictions
        self.transactionTypeRestrictions = transactionTypeRestrictions


    @classmethod
    def load_from_binary(cls, payload: bytes) -> AccountRestrictionsInfoBuilder:
        """Creates an instance of AccountRestrictionsInfoBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AccountRestrictionsInfoBuilder.
        """
        bytes_ = bytes(payload)

        restrictionFlags = AccountRestrictionFlagsDto.bytesToFlags(bytes_, 2)  # kind:FLAGS
        bytes_ = bytes_[2:]
        addressRestrictions = None
        if AccountRestrictionFlagsDto.ADDRESS in restrictionFlags:
            addressRestrictions = AccountRestrictionAddressValueBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[addressRestrictions.get_size():]
        mosaicIdRestrictions = None
        if AccountRestrictionFlagsDto.MOSAIC_ID in restrictionFlags:
            mosaicIdRestrictions = AccountRestrictionMosaicValueBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[mosaicIdRestrictions.get_size():]
        transactionTypeRestrictions = None
        if AccountRestrictionFlagsDto.TRANSACTION_TYPE in restrictionFlags:
            transactionTypeRestrictions = AccountRestrictionTransactionTypeValueBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[transactionTypeRestrictions.get_size():]
        return AccountRestrictionsInfoBuilder(restrictionFlags, addressRestrictions, mosaicIdRestrictions, transactionTypeRestrictions)

    def get_restriction_flags(self) -> List[AccountRestrictionFlagsDto]:
        """Gets raw restriction flags.
        Returns:
            Raw restriction flags.
        """
        return self.restrictionFlags

    def get_address_restrictions(self) -> AccountRestrictionAddressValueBuilder:
        """Gets address restrictions.
        Returns:
            Address restrictions.
        """
        if not AccountRestrictionFlagsDto.ADDRESS in self.restrictionFlags:
            raise Exception('restrictionFlags is not set to ADDRESS.')
        return self.addressRestrictions

    def get_mosaic_id_restrictions(self) -> AccountRestrictionMosaicValueBuilder:
        """Gets mosaic identifier restrictions.
        Returns:
            Mosaic identifier restrictions.
        """
        if not AccountRestrictionFlagsDto.MOSAIC_ID in self.restrictionFlags:
            raise Exception('restrictionFlags is not set to MOSAIC_ID.')
        return self.mosaicIdRestrictions

    def get_transaction_type_restrictions(self) -> AccountRestrictionTransactionTypeValueBuilder:
        """Gets transaction type restrictions.
        Returns:
            Transaction type restrictions.
        """
        if not AccountRestrictionFlagsDto.TRANSACTION_TYPE in self.restrictionFlags:
            raise Exception('restrictionFlags is not set to TRANSACTION_TYPE.')
        return self.transactionTypeRestrictions

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 2  # restrictionFlags
        if AccountRestrictionFlagsDto.ADDRESS in self.restrictionFlags:
            size += self.addressRestrictions.get_size()
        if AccountRestrictionFlagsDto.MOSAIC_ID in self.restrictionFlags:
            size += self.mosaicIdRestrictions.get_size()
        if AccountRestrictionFlagsDto.TRANSACTION_TYPE in self.restrictionFlags:
            size += self.transactionTypeRestrictions.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(AccountRestrictionFlagsDto.flagsToInt(self.get_restriction_flags()), 2))  # kind:FLAGS
        if AccountRestrictionFlagsDto.ADDRESS in self.restrictionFlags:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.addressRestrictions.serialize())  # kind:CUSTOM
        if AccountRestrictionFlagsDto.MOSAIC_ID in self.restrictionFlags:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.mosaicIdRestrictions.serialize())  # kind:CUSTOM
        if AccountRestrictionFlagsDto.TRANSACTION_TYPE in self.restrictionFlags:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.transactionTypeRestrictions.serialize())  # kind:CUSTOM
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
import re
from .GeneratorUtils import GeneratorUtils
from .UnresolvedAddressDto import UnresolvedAddressDto
from .UnresolvedMosaicIdDto import UnresolvedMosaicIdDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class MosaicAddressRestrictionTransactionBodyBuilder:
    """Binary layout for a mosaic address restriction transaction.

    Attributes:
        mosaic_id: Identifier of the mosaic to which the restriction applies.
        restriction_key: Restriction key.
        previous_restriction_value: Previous restriction value.
        new_restriction_value: New restriction value.
        target_address: Address being restricted.
    """
    def __init__(self):
        """ Constructor."""
        self.mosaic_id = UnresolvedMosaicIdDto().unresolvedMosaicId
        self.restriction_key = int()
        self.previous_restriction_value = int()
        self.new_restriction_value = int()
        self.target_address = bytes(24)

    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicAddressRestrictionTransactionBodyBuilder:
        """Creates an instance of MosaicAddressRestrictionTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicAddressRestrictionTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        mosaic_id_ = UnresolvedMosaicIdDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        mosaic_id = mosaic_id_.unresolvedMosaicId
        bytes_ = bytes_[mosaic_id_.get_size():]
        restriction_key = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        previous_restriction_value = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        new_restriction_value = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIMPLE
        bytes_ = bytes_[8:]
        target_address_ = UnresolvedAddressDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        target_address = target_address_.unresolvedAddress
        bytes_ = bytes_[target_address_.get_size():]

        # create object and call
        result = MosaicAddressRestrictionTransactionBodyBuilder()
        result.mosaic_id = mosaic_id
        result.restriction_key = restriction_key
        result.previous_restriction_value = previous_restriction_value
        result.new_restriction_value = new_restriction_value
        result.target_address = target_address
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += UnresolvedMosaicIdDto(self.mosaic_id).get_size()
        size += 8  # restriction_key
        size += 8  # previous_restriction_value
        size += 8  # new_restriction_value
        size += UnresolvedAddressDto(self.target_address).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedMosaicIdDto(self.mosaic_id).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.restriction_key, 8))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.previous_restriction_value, 8))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.new_restriction_value, 8))  # serial_kind:SIMPLE
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedAddressDto(self.target_address).serialize())  # kind:CUSTOM
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('mosaic_id', to_hex_string(UnresolvedMosaicIdDto(self.mosaic_id).serialize()))
        result += '{:24s} : {}\n'.format('restriction_key', to_hex_string(GeneratorUtils.uint_to_buffer(self.restriction_key, 8)))
        result += '{:24s} : {}\n'.format('previous_restriction_value', to_hex_string(GeneratorUtils.uint_to_buffer(self.previous_restriction_value, 8)))
        result += '{:24s} : {}\n'.format('new_restriction_value', to_hex_string(GeneratorUtils.uint_to_buffer(self.new_restriction_value, 8)))
        result += '{:24s} : {}\n'.format('target_address', to_hex_string(UnresolvedAddressDto(self.target_address).serialize()))
        return result



from setuptools import setup, find_packages  # noqa: H301

with open('README.md', 'r') as readme_file:
    README = readme_file.read()

NAME = 'catbuffer'
VERSION = '1.0.0.20210408.135739a1'

REQUIRES = []

setup(
    name=NAME,
    version=VERSION,
    description='Symbol Catbuffer Builders',
    author='NEM Group',
    author_email='dev@nem.software',
    url='https://github.com/nemtech/catbuffer-generators',
    keywords=['catbuffer-generators', 'catbuffer', 'builders', 'Symbol Catbuffer Builders'],
    install_requires=REQUIRES,
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    license='Apache 2.0',
    long_description=README,
    long_description_content_type='text/markdown',
    classifiers=[
        'Programming Language :: Python :: 3.7',
    ]
)



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
from typing import List
import re
from .GeneratorUtils import GeneratorUtils
from .AccountRestrictionFlagsDto import AccountRestrictionFlagsDto
from .EntityTypeDto import EntityTypeDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class AccountOperationRestrictionTransactionBodyBuilder:
    """Binary layout for an account operation restriction transaction.

    Attributes:
        restriction_flags: Account restriction flags.
        restriction_additions: Account restriction additions.
        restriction_deletions: Account restriction deletions.
    """
    def __init__(self):
        """ Constructor."""
        self.restriction_flags = []
        self.restriction_additions = []
        self.restriction_deletions = []

    @classmethod
    def load_from_binary(cls, payload: bytes) -> AccountOperationRestrictionTransactionBodyBuilder:
        """Creates an instance of AccountOperationRestrictionTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AccountOperationRestrictionTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        restriction_flags = AccountRestrictionFlagsDto.bytesToFlags(bytes_, 2)  # kind:FLAGS
        bytes_ = bytes_[2:]
        restriction_additions_count = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIZE_FIELD
        bytes_ = bytes_[1:]
        restriction_deletions_count = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIZE_FIELD
        bytes_ = bytes_[1:]
        account_restriction_transaction_body__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        restriction_additions = []  # kind:ARRAY
        for _ in range(restriction_additions_count):
            item = EntityTypeDto.load_from_binary(bytes_)
            restriction_additions.append(item)
            bytes_ = bytes_[item.get_size():]
        restriction_deletions = []  # kind:ARRAY
        for _ in range(restriction_deletions_count):
            item = EntityTypeDto.load_from_binary(bytes_)
            restriction_deletions.append(item)
            bytes_ = bytes_[item.get_size():]

        # create object and call
        result = AccountOperationRestrictionTransactionBodyBuilder()
        result.restriction_flags = restriction_flags
        result.restriction_additions = restriction_additions
        result.restriction_deletions = restriction_deletions
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 2  # restriction_flags
        size += 1  # restriction_additions_count
        size += 1  # restriction_deletions_count
        size += 4  # account_restriction_transaction_body__reserved1
        for _ in self.restriction_additions:
            size += EntityTypeDto(_).get_size()
        for _ in self.restriction_deletions:
            size += EntityTypeDto(_).get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(AccountRestrictionFlagsDto.flagsToInt(self.restriction_flags), 2))  # kind:FLAGS
        size_value = len(self.restriction_additions)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 1))  # kind:SIZE_FIELD
        size_value = len(self.restriction_deletions)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 1))  # kind:SIZE_FIELD
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        for _ in self.restriction_additions: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, EntityTypeDto(_).serialize())
        for _ in self.restriction_deletions: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, EntityTypeDto(_).serialize())
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        _serializedFlags = GeneratorUtils.uint_to_buffer(AccountRestrictionFlagsDto.flagsToInt(self.restriction_flags), 2)
        result += '{:24s} : {} {}\n'.format('restriction_flags', to_hex_string(_serializedFlags), self.restriction_flags)
        size_value = len(self.restriction_additions)
        result += '{:24s} : {}\n'.format('restriction_additions_count', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 1)))
        size_value = len(self.restriction_deletions)
        result += '{:24s} : {}\n'.format('restriction_deletions_count', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 1)))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        result += '{:24s} : [\n'.format('restriction_additions')
        for _ in self.restriction_additions: # kind:ARRAY|FILL_ARRAY
            result += '  {}\n'.format(EntityTypeDto(_).__str__())
        result += ']\n'
        result += '{:24s} : [\n'.format('restriction_deletions')
        for _ in self.restriction_deletions: # kind:ARRAY|FILL_ARRAY
            result += '  {}\n'.format(EntityTypeDto(_).__str__())
        result += ']\n'
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .MosaicIdDto import MosaicIdDto

class AccountRestrictionMosaicValueBuilder:
    """Binary layout for mosaic id based account restriction.

    Attributes:
        restrictionValues: Restriction values.
    """

    def __init__(self, restrictionValues: List[MosaicIdDto]):
        """Constructor.
        Args:
            restrictionValues: Restriction values.
        """
        self.restrictionValues = restrictionValues


    @classmethod
    def load_from_binary(cls, payload: bytes) -> AccountRestrictionMosaicValueBuilder:
        """Creates an instance of AccountRestrictionMosaicValueBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AccountRestrictionMosaicValueBuilder.
        """
        bytes_ = bytes(payload)

        restrictionValuesCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 8))  # kind:SIZE_FIELD
        bytes_ = bytes_[8:]
        restrictionValues: List[MosaicIdDto] = []  # kind:ARRAY
        for _ in range(restrictionValuesCount):
            item = MosaicIdDto.load_from_binary(bytes_)
            restrictionValues.append(item)
            bytes_ = bytes_[item.get_size():]
        return AccountRestrictionMosaicValueBuilder(restrictionValues)

    def get_restriction_values(self) -> List[MosaicIdDto]:
        """Gets restriction values.
        Returns:
            Restriction values.
        """
        return self.restrictionValues

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += 8  # restrictionValuesCount
        for _ in self.restrictionValues:
            size += _.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(len(self.get_restriction_values()), 8))  # kind:SIZE_FIELD
        for _ in self.restrictionValues: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from typing import List
from .GeneratorUtils import GeneratorUtils
from .AccountKeyTypeFlagsDto import AccountKeyTypeFlagsDto
from .AccountStateFormatDto import AccountStateFormatDto
from .AccountTypeDto import AccountTypeDto
from .AddressDto import AddressDto
from .HeightActivityBucketsBuilder import HeightActivityBucketsBuilder
from .HeightDto import HeightDto
from .ImportanceSnapshotBuilder import ImportanceSnapshotBuilder
from .KeyDto import KeyDto
from .MosaicBuilder import MosaicBuilder
from .PinnedVotingKeyBuilder import PinnedVotingKeyBuilder
from .StateHeaderBuilder import StateHeaderBuilder

class AccountStateBuilder(StateHeaderBuilder):
    """Binary layout for non-historical account state.

    Attributes:
        address: Address of account.
        addressHeight: Height at which address has been obtained.
        publicKey: Public key of account.
        publicKeyHeight: Height at which public key has been obtained.
        accountType: Type of account.
        format: Account format.
        supplementalPublicKeysMask: Mask of supplemental public key flags.
        linkedPublicKey: Linked account public key.
        nodePublicKey: Node public key.
        vrfPublicKey: Vrf public key.
        votingPublicKeys: Voting public keys.
        importanceSnapshots: Current importance snapshot of the account.
        activityBuckets: Activity buckets of the account.
        balances: Balances of account.
    """

    def __init__(self, version: int, address: AddressDto, addressHeight: HeightDto, publicKey: KeyDto, publicKeyHeight: HeightDto, accountType: AccountTypeDto, format: AccountStateFormatDto, supplementalPublicKeysMask: List[AccountKeyTypeFlagsDto], linkedPublicKey: KeyDto, nodePublicKey: KeyDto, vrfPublicKey: KeyDto, votingPublicKeys: List[PinnedVotingKeyBuilder], importanceSnapshots: ImportanceSnapshotBuilder, activityBuckets: HeightActivityBucketsBuilder, balances: List[MosaicBuilder]):
        """Constructor.
        Args:
            version: Serialization version.
            address: Address of account.
            addressHeight: Height at which address has been obtained.
            publicKey: Public key of account.
            publicKeyHeight: Height at which public key has been obtained.
            accountType: Type of account.
            format: Account format.
            supplementalPublicKeysMask: Mask of supplemental public key flags.
            linkedPublicKey: Linked account public key.
            nodePublicKey: Node public key.
            vrfPublicKey: Vrf public key.
            votingPublicKeys: Voting public keys.
            importanceSnapshots: Current importance snapshot of the account.
            activityBuckets: Activity buckets of the account.
            balances: Balances of account.
        """
        super().__init__(version)
        self.address = address
        self.addressHeight = addressHeight
        self.publicKey = publicKey
        self.publicKeyHeight = publicKeyHeight
        self.accountType = accountType
        self.format = format
        self.supplementalPublicKeysMask = supplementalPublicKeysMask
        self.linkedPublicKey = linkedPublicKey
        self.nodePublicKey = nodePublicKey
        self.vrfPublicKey = vrfPublicKey
        self.votingPublicKeys = votingPublicKeys
        self.importanceSnapshots = importanceSnapshots
        self.activityBuckets = activityBuckets
        self.balances = balances


    @classmethod
    def load_from_binary(cls, payload: bytes) -> AccountStateBuilder:
        """Creates an instance of AccountStateBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of AccountStateBuilder.
        """
        bytes_ = bytes(payload)
        superObject = StateHeaderBuilder.load_from_binary(bytes_)
        bytes_ = bytes_[superObject.get_size():]

        address = AddressDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[address.get_size():]
        addressHeight = HeightDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[addressHeight.get_size():]
        publicKey = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[publicKey.get_size():]
        publicKeyHeight = HeightDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[publicKeyHeight.get_size():]
        accountType = AccountTypeDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[accountType.get_size():]
        format = AccountStateFormatDto.load_from_binary(bytes_)  # kind:CUSTOM2
        bytes_ = bytes_[format.get_size():]
        supplementalPublicKeysMask = AccountKeyTypeFlagsDto.bytesToFlags(bytes_, 1)  # kind:FLAGS
        bytes_ = bytes_[1:]
        votingPublicKeysCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIZE_FIELD
        bytes_ = bytes_[1:]
        linkedPublicKey = None
        if AccountKeyTypeFlagsDto.LINKED in supplementalPublicKeysMask:
            linkedPublicKey = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[linkedPublicKey.get_size():]
        nodePublicKey = None
        if AccountKeyTypeFlagsDto.NODE in supplementalPublicKeysMask:
            nodePublicKey = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[nodePublicKey.get_size():]
        vrfPublicKey = None
        if AccountKeyTypeFlagsDto.VRF in supplementalPublicKeysMask:
            vrfPublicKey = KeyDto.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[vrfPublicKey.get_size():]
        votingPublicKeys: List[PinnedVotingKeyBuilder] = []  # kind:ARRAY
        for _ in range(votingPublicKeysCount):
            item = PinnedVotingKeyBuilder.load_from_binary(bytes_)
            votingPublicKeys.append(item)
            bytes_ = bytes_[item.get_size():]
        importanceSnapshots = None
        if format == AccountStateFormatDto.HIGH_VALUE:
            importanceSnapshots = ImportanceSnapshotBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[importanceSnapshots.get_size():]
        activityBuckets = None
        if format == AccountStateFormatDto.HIGH_VALUE:
            activityBuckets = HeightActivityBucketsBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
            bytes_ = bytes_[activityBuckets.get_size():]
        balancesCount = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 2))  # kind:SIZE_FIELD
        bytes_ = bytes_[2:]
        balances: List[MosaicBuilder] = []  # kind:ARRAY
        for _ in range(balancesCount):
            item = MosaicBuilder.load_from_binary(bytes_)
            balances.append(item)
            bytes_ = bytes_[item.get_size():]
        return AccountStateBuilder(superObject.version, address, addressHeight, publicKey, publicKeyHeight, accountType, format, supplementalPublicKeysMask, linkedPublicKey, nodePublicKey, vrfPublicKey, votingPublicKeys, importanceSnapshots, activityBuckets, balances)

    def get_address(self) -> AddressDto:
        """Gets address of account.
        Returns:
            Address of account.
        """
        return self.address

    def get_address_height(self) -> HeightDto:
        """Gets height at which address has been obtained.
        Returns:
            Height at which address has been obtained.
        """
        return self.addressHeight

    def get_public_key(self) -> KeyDto:
        """Gets public key of account.
        Returns:
            Public key of account.
        """
        return self.publicKey

    def get_public_key_height(self) -> HeightDto:
        """Gets height at which public key has been obtained.
        Returns:
            Height at which public key has been obtained.
        """
        return self.publicKeyHeight

    def get_account_type(self) -> AccountTypeDto:
        """Gets type of account.
        Returns:
            Type of account.
        """
        return self.accountType

    def get_format(self) -> AccountStateFormatDto:
        """Gets account format.
        Returns:
            Account format.
        """
        return self.format

    def get_supplemental_public_keys_mask(self) -> List[AccountKeyTypeFlagsDto]:
        """Gets mask of supplemental public key flags.
        Returns:
            Mask of supplemental public key flags.
        """
        return self.supplementalPublicKeysMask

    def get_linked_public_key(self) -> KeyDto:
        """Gets linked account public key.
        Returns:
            Linked account public key.
        """
        if not AccountKeyTypeFlagsDto.LINKED in self.supplementalPublicKeysMask:
            raise Exception('supplementalPublicKeysMask is not set to LINKED.')
        return self.linkedPublicKey

    def get_node_public_key(self) -> KeyDto:
        """Gets node public key.
        Returns:
            Node public key.
        """
        if not AccountKeyTypeFlagsDto.NODE in self.supplementalPublicKeysMask:
            raise Exception('supplementalPublicKeysMask is not set to NODE.')
        return self.nodePublicKey

    def get_vrf_public_key(self) -> KeyDto:
        """Gets vrf public key.
        Returns:
            Vrf public key.
        """
        if not AccountKeyTypeFlagsDto.VRF in self.supplementalPublicKeysMask:
            raise Exception('supplementalPublicKeysMask is not set to VRF.')
        return self.vrfPublicKey

    def get_voting_public_keys(self) -> List[PinnedVotingKeyBuilder]:
        """Gets voting public keys.
        Returns:
            Voting public keys.
        """
        return self.votingPublicKeys

    def get_importance_snapshots(self) -> ImportanceSnapshotBuilder:
        """Gets current importance snapshot of the account.
        Returns:
            Current importance snapshot of the account.
        """
        if not self.format == AccountStateFormatDto.HIGH_VALUE:
            raise Exception('format is not set to HIGH_VALUE.')
        return self.importanceSnapshots

    def get_activity_buckets(self) -> HeightActivityBucketsBuilder:
        """Gets activity buckets of the account.
        Returns:
            Activity buckets of the account.
        """
        if not self.format == AccountStateFormatDto.HIGH_VALUE:
            raise Exception('format is not set to HIGH_VALUE.')
        return self.activityBuckets

    def get_balances(self) -> List[MosaicBuilder]:
        """Gets balances of account.
        Returns:
            Balances of account.
        """
        return self.balances

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = super().get_size()
        size += self.address.get_size()
        size += self.addressHeight.get_size()
        size += self.publicKey.get_size()
        size += self.publicKeyHeight.get_size()
        size += self.accountType.get_size()
        size += self.format.get_size()
        size += 1  # supplementalPublicKeysMask
        size += 1  # votingPublicKeysCount
        if AccountKeyTypeFlagsDto.LINKED in self.supplementalPublicKeysMask:
            size += self.linkedPublicKey.get_size()
        if AccountKeyTypeFlagsDto.NODE in self.supplementalPublicKeysMask:
            size += self.nodePublicKey.get_size()
        if AccountKeyTypeFlagsDto.VRF in self.supplementalPublicKeysMask:
            size += self.vrfPublicKey.get_size()
        for _ in self.votingPublicKeys:
            size += _.get_size()
        if self.format == AccountStateFormatDto.HIGH_VALUE:
            size += self.importanceSnapshots.get_size()
        if self.format == AccountStateFormatDto.HIGH_VALUE:
            size += self.activityBuckets.get_size()
        size += 2  # balancesCount
        for _ in self.balances:
            size += _.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, super().serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.address.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.addressHeight.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.publicKey.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.publicKeyHeight.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.accountType.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.format.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(AccountKeyTypeFlagsDto.flagsToInt(self.get_supplemental_public_keys_mask()), 1))  # kind:FLAGS
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(len(self.get_voting_public_keys()), 1))  # kind:SIZE_FIELD
        if AccountKeyTypeFlagsDto.LINKED in self.supplementalPublicKeysMask:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.linkedPublicKey.serialize())  # kind:CUSTOM
        if AccountKeyTypeFlagsDto.NODE in self.supplementalPublicKeysMask:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.nodePublicKey.serialize())  # kind:CUSTOM
        if AccountKeyTypeFlagsDto.VRF in self.supplementalPublicKeysMask:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.vrfPublicKey.serialize())  # kind:CUSTOM
        for _ in self.votingPublicKeys: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        if self.format == AccountStateFormatDto.HIGH_VALUE:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.importanceSnapshots.serialize())  # kind:CUSTOM
        if self.format == AccountStateFormatDto.HIGH_VALUE:
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.activityBuckets.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(len(self.get_balances()), 2))  # kind:SIZE_FIELD
        for _ in self.balances: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, _.serialize())
        return bytes_
    # end of class



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
from typing import List
import re
from .GeneratorUtils import GeneratorUtils
from .BlockDurationDto import BlockDurationDto
from .MosaicFlagsDto import MosaicFlagsDto
from .MosaicIdDto import MosaicIdDto
from .MosaicNonceDto import MosaicNonceDto

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class MosaicDefinitionTransactionBodyBuilder:
    """Binary layout for a mosaic definition transaction.

    Attributes:
        id: Mosaic identifier.
        duration: Mosaic duration.
        nonce: Mosaic nonce.
        flags: Mosaic flags.
        divisibility: Mosaic divisibility.
    """
    def __init__(self):
        """ Constructor."""
        self.id = MosaicIdDto().mosaicId
        self.duration = BlockDurationDto().blockDuration
        self.nonce = MosaicNonceDto().mosaicNonce
        self.flags = []
        self.divisibility = int()

    @classmethod
    def load_from_binary(cls, payload: bytes) -> MosaicDefinitionTransactionBodyBuilder:
        """Creates an instance of MosaicDefinitionTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of MosaicDefinitionTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        id_ = MosaicIdDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        id = id_.mosaicId
        bytes_ = bytes_[id_.get_size():]
        duration_ = BlockDurationDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        duration = duration_.blockDuration
        bytes_ = bytes_[duration_.get_size():]
        nonce_ = MosaicNonceDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        nonce = nonce_.mosaicNonce
        bytes_ = bytes_[nonce_.get_size():]
        flags = MosaicFlagsDto.bytesToFlags(bytes_, 1)  # kind:FLAGS
        bytes_ = bytes_[1:]
        divisibility = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]

        # create object and call
        result = MosaicDefinitionTransactionBodyBuilder()
        result.id = id
        result.duration = duration
        result.nonce = nonce
        result.flags = flags
        result.divisibility = divisibility
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += MosaicIdDto(self.id).get_size()
        size += BlockDurationDto(self.duration).get_size()
        size += MosaicNonceDto(self.nonce).get_size()
        size += 1  # flags
        size += 1  # divisibility
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, MosaicIdDto(self.id).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, BlockDurationDto(self.duration).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, MosaicNonceDto(self.nonce).serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(MosaicFlagsDto.flagsToInt(self.flags), 1))  # kind:FLAGS
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(self.divisibility, 1))  # serial_kind:SIMPLE
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('id', to_hex_string(MosaicIdDto(self.id).serialize()))
        result += '{:24s} : {}\n'.format('duration', to_hex_string(BlockDurationDto(self.duration).serialize()))
        result += '{:24s} : {}\n'.format('nonce', to_hex_string(MosaicNonceDto(self.nonce).serialize()))
        _serializedFlags = GeneratorUtils.uint_to_buffer(MosaicFlagsDto.flagsToInt(self.flags), 1)
        result += '{:24s} : {} {}\n'.format('flags', to_hex_string(_serializedFlags), self.flags)
        result += '{:24s} : {}\n'.format('divisibility', to_hex_string(GeneratorUtils.uint_to_buffer(self.divisibility, 1)))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations

# pylint: disable=unused-import

from binascii import hexlify
from typing import List
import re
from .GeneratorUtils import GeneratorUtils
from .UnresolvedAddressDto import UnresolvedAddressDto
from .UnresolvedMosaicBuilder import UnresolvedMosaicBuilder

def to_hex_string(bin):
    return hexlify(bin).decode('utf-8')

class TransferTransactionBodyBuilder:
    """Binary layout for a transfer transaction.

    Attributes:
        recipient_address: Recipient address.
        mosaics: Attached mosaics.
        message: Attached message.
    """
    def __init__(self):
        """ Constructor."""
        self.recipient_address = bytes(24)
        self.mosaics = []
        self.message = bytes()

    @classmethod
    def load_from_binary(cls, payload: bytes) -> TransferTransactionBodyBuilder:
        """Creates an instance of TransferTransactionBodyBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of TransferTransactionBodyBuilder.
        """
        bytes_ = bytes(payload)

        recipient_address_ = UnresolvedAddressDto.load_from_binary(bytes_)  # kind:CUSTOM1_byte
        recipient_address = recipient_address_.unresolvedAddress
        bytes_ = bytes_[recipient_address_.get_size():]
        message_size = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 2))  # kind:SIZE_FIELD
        bytes_ = bytes_[2:]
        mosaics_count = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIZE_FIELD
        bytes_ = bytes_[1:]
        transfer_transaction_body__reserved1 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 4))  # kind:SIMPLE
        bytes_ = bytes_[4:]
        transfer_transaction_body__reserved2 = GeneratorUtils.buffer_to_uint(GeneratorUtils.get_bytes(bytes_, 1))  # kind:SIMPLE
        bytes_ = bytes_[1:]
        mosaics = []  # kind:ARRAY
        for _ in range(mosaics_count):
            item = UnresolvedMosaicBuilder.load_from_binary(bytes_)
            mosaics.append(item.as_tuple())
            bytes_ = bytes_[item.get_size():]
        message = GeneratorUtils.get_bytes(bytes_, message_size)  # kind:BUFFER
        bytes_ = bytes_[message_size:]

        # create object and call
        result = TransferTransactionBodyBuilder()
        result.recipient_address = recipient_address
        result.mosaics = mosaics
        result.message = message
        return result

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += UnresolvedAddressDto(self.recipient_address).get_size()
        size += 2  # message_size
        size += 1  # mosaics_count
        size += 4  # transfer_transaction_body__reserved1
        size += 1  # transfer_transaction_body__reserved2
        for _ in self.mosaics:
            size += UnresolvedMosaicBuilder.from_tuple(_).get_size()
        size += len(self.message)
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedAddressDto(self.recipient_address).serialize())  # kind:CUSTOM
        size_value = len(self.message)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 2))  # kind:SIZE_FIELD
        size_value = len(self.mosaics)
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(size_value, 1))  # kind:SIZE_FIELD
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 4))
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, GeneratorUtils.uint_to_buffer(0, 1))
        for _ in self.mosaics: # kind:ARRAY|FILL_ARRAY
            bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, UnresolvedMosaicBuilder.from_tuple(_).serialize())
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.message)  # kind:BUFFER
        return bytes_

    def __str__(self):
        """Returns nice representation.
        Returns:
            Printable string
        """
        result = ''
        result += '{:24s} : {}\n'.format('recipient_address', to_hex_string(UnresolvedAddressDto(self.recipient_address).serialize()))
        size_value = len(self.message)
        result += '{:24s} : {}\n'.format('message_size', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 2)))
        size_value = len(self.mosaics)
        result += '{:24s} : {}\n'.format('mosaics_count', to_hex_string(GeneratorUtils.uint_to_buffer(size_value, 1)))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 4)))
        result += '{:24s} : {}\n'.format('<reserved>', to_hex_string(GeneratorUtils.uint_to_buffer(0, 1)))
        result += '{:24s} : [\n'.format('mosaics')
        for _ in self.mosaics: # kind:ARRAY|FILL_ARRAY
            result += '  {}\n'.format(UnresolvedMosaicBuilder.from_tuple(_).__str__())
        result += ']\n'
        result += '{:24s} : {}\n'.format('message', to_hex_string(self.message))
        return result



#!/usr/bin/python
"""
    Copyright (c) 2016-2019, Jaguar0625, gimre, BloodyRookie, Tech Bureau, Corp.
    Copyright (c) 2020-present, Jaguar0625, gimre, BloodyRookie.

    This file is part of Catapult.

    Catapult is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Catapult is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Catapult. If not, see <http://www.gnu.org/licenses/>.
"""

# pylint: disable=W0622,W0612,C0301,R0904

from __future__ import annotations
from .GeneratorUtils import GeneratorUtils
from .FinalizationRoundBuilder import FinalizationRoundBuilder
from .Hash256Dto import Hash256Dto
from .HeightDto import HeightDto

class FinalizedBlockHeaderBuilder:
    """Binary layout for finalized block header.

    Attributes:
        round: Finalization round.
        height: Finalization height.
        hash: Finalization hash.
    """

    def __init__(self, round: FinalizationRoundBuilder, height: HeightDto, hash: Hash256Dto):
        """Constructor.
        Args:
            round: Finalization round.
            height: Finalization height.
            hash: Finalization hash.
        """
        self.round = round
        self.height = height
        self.hash = hash


    @classmethod
    def load_from_binary(cls, payload: bytes) -> FinalizedBlockHeaderBuilder:
        """Creates an instance of FinalizedBlockHeaderBuilder from binary payload.
        Args:
            payload: Byte payload to use to serialize the object.
        Returns:
            Instance of FinalizedBlockHeaderBuilder.
        """
        bytes_ = bytes(payload)

        round = FinalizationRoundBuilder.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[round.get_size():]
        height = HeightDto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[height.get_size():]
        hash = Hash256Dto.load_from_binary(bytes_)  # kind:CUSTOM1
        bytes_ = bytes_[hash.get_size():]
        return FinalizedBlockHeaderBuilder(round, height, hash)

    def get_round(self) -> FinalizationRoundBuilder:
        """Gets finalization round.
        Returns:
            Finalization round.
        """
        return self.round

    def get_height(self) -> HeightDto:
        """Gets finalization height.
        Returns:
            Finalization height.
        """
        return self.height

    def get_hash(self) -> Hash256Dto:
        """Gets finalization hash.
        Returns:
            Finalization hash.
        """
        return self.hash

    def get_size(self) -> int:
        """Gets the size of the object.
        Returns:
            Size in bytes.
        """
        size = 0
        size += self.round.get_size()
        size += self.height.get_size()
        size += self.hash.get_size()
        return size

    def serialize(self) -> bytes:
        """Serializes self to bytes.
        Returns:
            Serialized bytes.
        """
        bytes_ = bytes()
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.round.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.height.serialize())  # kind:CUSTOM
        bytes_ = GeneratorUtils.concat_typed_arrays(bytes_, self.hash.serialize())  # kind:CUSTOM
        return bytes_
    # end of class
